/***************************************************************************
                          klatinverbs.cpp  -  description
                             -------------------
    begin                : Fri Aug 15
    copyright            : (C) 2001-2004 by George Wright
    email                : gwright@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qlabel.h>

#include <kapplication.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>

#include <stdlib.h>
#include <time.h>

#include "klatinverbs.h"

KLatinVerbs::KLatinVerbs(QWidget* parent, const char *name)
	: KLatinVerbsWidget(parent, name)
{
	ChooseConjugation->insertItem(i18n("1st"));
	ChooseConjugation->insertItem(i18n("2nd"));
	ChooseConjugation->insertItem(i18n("3rd"));
	ChooseConjugation->insertItem(i18n("4th"));
	ChooseConjugation->insertItem("sum"); // Does not need translating as it's in Latin

	ChooseMood->insertItem(i18n("Indicative"));
	ChooseMood->insertItem(i18n("Subjunctive"));

	ChooseVoice->insertItem(i18n("Active"));
	ChooseVoice->insertItem(i18n("Passive"));

	m_total = 0;
	m_totalCorrect = 0;
	m_numWrong = 0;

	connect(ChooseConjugation, SIGNAL(activated(int)), this, SLOT(conjugationActivated(int)));
	connect(BackButton, SIGNAL(clicked()), this, SLOT(goBack()));
	connect(ChangeButton, SIGNAL(clicked()), this, SLOT(loadDatabase()));
	connect(AnswerBox, SIGNAL(returnPressed()), this, SLOT(checkAnswer()));
	connect(OKButton, SIGNAL(clicked()), this, SLOT(checkAnswer()));

	loadDatabase();
}

KLatinVerbs::~KLatinVerbs()
{
}

void KLatinVerbs::loadDatabase()
{
	QString databaseFile;

	// These are words in Latin and do not need i18n
	name.declension[0] = "amo";
	name.declension[1] = "moneo";
	name.declension[2] = "rego";
	name.declension[3] = "audio";
	name.declension[4] = "sum";
	
	name.mood[0] = i18n("indicative");
	name.mood[1] = i18n("subjunctive");
	
	name.voice[0] = i18n("active");
	name.voice[1] = i18n("passive");
	
	databaseFile = name.declension[ChooseConjugation->currentItem()];
	databaseFile += "-" + name.mood[ChooseMood->currentItem()];
	databaseFile += "-" + name.voice[ChooseVoice->currentItem()] + ".klat";
	
	QString fileName = "klatin/data/verbs/";
	fileName += databaseFile;

	QFile check(locate("data", fileName));

	if (check.exists() == FALSE) {
		check.close();
		KMessageBox::error(0, i18n("Could not load verb-definition file, check KLatin installation."), i18n("Error with KLatin"));
		goBack();
	}

	int numberOfWords = 0;

	KEduVocDataItemList verbs = KEduVocData::parse(locate("data", fileName));

	for(KEduVocDataItemList::Iterator dataIt = verbs.begin(); dataIt != verbs.end(); dataIt++) {
			m_verb[numberOfWords] = verbs[numberOfWords].originalText();
			numberOfWords++;
	}

	nextQuestion();
}

void KLatinVerbs::nextQuestion()
{
	struct questionparts question;
	question.person[0] = i18n("indicates the first person", "first");
	question.person[1] = i18n("indicates the second person", "second");
	question.person[2] = i18n("indicates the third person", "third");
	
	question.number[0] = i18n("singular");
	question.number[1] = i18n("plural");
	
	question.tense[0] = i18n("present");
	question.tense[1] = i18n("future");
	question.tense[2] = i18n("imperfect");
	question.tense[3] = i18n("perfect");
	question.tense[4] = i18n("pluperfect");
	question.tense[5] = i18n("future perfect");
	
	// Seed the random number generator
	srandom(time(NULL));

	if (ChooseMood->currentText() == i18n("Indicative"))
		m_verbNumber = (random() % 35) + 1;
	if (ChooseMood->currentText() == i18n("Subjunctive"))
		m_verbNumber = (random() % 23) + 1;

	QString QuestionString;

	int person = 0, number = 0, tense = 0;
	
	switch (m_verbNumber % 6) {
		case 1:
			person = 0;
			number = 0;
			break;
		case 2:
			person = 1;
			number = 0;
			break;
		case 3:
			person = 2;
			number = 0;
			break;
		case 4:
			person = 0;
			number = 1;
			break;
		case 5:
			person = 1;
			number = 1;
			break;
		case 0:
			person = 2;
			number = 1;
			break;
	}
	
	switch ((int) (m_verbNumber - 1) / 6) {
		case 0:
			tense = 0;
			break;
		case 1:
			if (ChooseMood->currentText() == i18n("Indicative"))
				tense = 1;
			else
				tense = 2;
			break;
		case 2:
			if (ChooseMood->currentText() == i18n("Indicative"))
				tense = 2;
			else
				tense = 3;
			break;
		case 3:
			if (ChooseMood->currentText() == i18n("Indicative"))
				tense = 3;
			else
				tense = 4;
			break;
		case 4:
			tense = 4;
			break;
		case 5:
			tense = 5;
			break;
		}
		
	QuestionString = i18n("Note: ARGUMENT %1 is an ordinal number (1st, 2nd, 3rd)."
		"ARGUMENT %2 is the tense of the verb (Present, Imperfect, Perfect)."
		"ARGUMENT %3 is the mood of the verb (Indicative, Subjunctive)."
		"ARGUMENT %4 is the voice of the verb (Active, Passive)."
		"ARGUMENT %5 is the verb name (amo, moneo etc). This is in Latin and should not be translated."
		"The output string in English is something like: What is the first person singular present indicative active of amo?", 
		"What is the %1 person %2 %3 %4 %5 of %6?").arg(question.person[person])
		.arg(question.number[number])
		.arg(question.tense[tense])
		.arg(name.mood[ChooseMood->currentItem()])
		.arg(name.voice[ChooseVoice->currentItem()])
		.arg(name.declension[ChooseConjugation->currentItem()]);
	
	QuestionLabel->setText(QuestionString);
}

void KLatinVerbs::checkAnswer()
{
	if (AnswerBox->text() == "") {
		KMessageBox::information(0, i18n("Inform the user that nothing has been typed in", "Please enter an answer."), i18n("Error"));
		return;
	}
	
	if (m_verb[m_verbNumber - 1] != AnswerBox->text().lower()) {
		// Iterate...
		m_total++;
		m_numWrong++;

		// Retry tolerance...
		if (m_numWrong == 2) {
			m_numWrong = 0;
			KMessageBox::information(0, i18n("The correct answer was %1.").arg(m_verb[m_verbNumber - 1]), i18n("Wrong"));
			nextQuestion();
		}

		AnswerBox->setText(NULL);
	}
	if (m_verb[m_verbNumber - 1] == AnswerBox->text().lower()) {
		// Iterate...
		m_totalCorrect++;
		m_total++;

		m_numWrong = 0;
		AnswerBox->setText(NULL);

		nextQuestion();
	}

	updateScores();
}

void KLatinVerbs::conjugationActivated(int itemNumber)
{
	if (itemNumber == 4)
		ChooseVoice->removeItem(1);
	if (itemNumber != 4)
		if (ChooseVoice->count() == 1)
			ChooseVoice->insertItem(i18n("Passive"));
}

void KLatinVerbs::updateScores()
{
	// Update the score labels
	Score1Label->setNum(m_totalCorrect);
	Score2Label->setNum(m_total);

	// Set percentage
	QString temp;
	temp.setNum(int(m_totalCorrect / m_total * 100));
	temp.insert(temp.length(), "%");
	PercentageLabel->setText(temp);
}

void KLatinVerbs::goBack()
{
	emit exited();
	close();
}

#include "klatinverbs.moc"
